// FSK441Gen.cpp: implementation of the CFSK441Gen class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "pathsim.h"
#include "FSK441Gen.h"
#include <math.h>
#include "tables.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define SYMBOL_PERIOD (25.0/11025.0)	//441 sym/sec
#define SYM0_FREQ (882.0)
#define SYM1_FREQ (1323.0)
#define SYM2_FREQ (1764.0)
#define SYM3_FREQ (2205.0)
#define SYM_PER_CHAR 3

#define K_2PI ( 8.0 * atan(1.0) )		// 2 Pi


#define TX_CONSTANT 20000.0		// TX Amplitude Factor


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFSK441Gen::CFSK441Gen()
{

}

CFSK441Gen::~CFSK441Gen()
{

}

void CFSK441Gen::Init( double Fs, CString Text)
{
	m_CarrierPhaseInc = SYM0_FREQ*K_2PI/Fs;
	m_SecPerSamp = 1.0/Fs;
	m_PhaseIncTbl[0] = SYM0_FREQ*K_2PI/Fs;
	m_PhaseIncTbl[1] = SYM1_FREQ*K_2PI/Fs;
	m_PhaseIncTbl[2] = SYM2_FREQ*K_2PI/Fs;
	m_PhaseIncTbl[3] = SYM3_FREQ*K_2PI/Fs;
	m_time = 0.0;
	m_SymbTime = SYMBOL_PERIOD;
	m_TextString = Text;
	if( (m_TextString.Find('+') == -1) &&
		(m_TextString.Find('*') == -1) &&
		(m_TextString.Find('%') == -1) &&
		(m_TextString.Find('@') == -1) )
	{
		m_TextString = m_TextString + ' ';
	}
	m_TextLength = m_TextString.GetLength();
	m_TextPos = 0;
	m_SymCount = 3;
}

/////////////////////////////////////////////////////////////
// generates n samples of FSK441 waveform in data array pData
/////////////////////////////////////////////////////////////
void CFSK441Gen::CalcFSK441(double *pData, INT n)
{
INT i;
	for( i=0; i<n; i++ )		//calculate n samples of tx data stream
	{
		pData[i] = TX_CONSTANT*sin( m_time );
		m_time += m_CarrierPhaseInc;		// increment radian phase count
		m_SymbTime += m_SecPerSamp;
		if( m_SymbTime >= SYMBOL_PERIOD )//if time to update symbol 
		{
			if( ++m_SymCount >= SYM_PER_CHAR)
			{
				m_SymAcc = FSK441_TABLE[GetChar()];
				m_SymCount = 0;
			}
			m_CarrierPhaseInc = m_PhaseIncTbl[m_SymAcc&0x03];
			m_SymAcc = m_SymAcc>>2;
			m_SymbTime -= SYMBOL_PERIOD;	//keep time bounded
			m_time = fmod(m_time,K_2PI);	//keep radian counter bounded
		}
	}
}


/////////////////////////////////////////////////////////////
// Retrieve nect text character
/////////////////////////////////////////////////////////////
INT CFSK441Gen::GetChar()
{
INT ch;
	if( m_TextPos >= m_TextLength)
		m_TextPos = 0;
	ch = m_TextString.GetAt(m_TextPos++) & 0x7F;
	if( (ch < ' ') || ( ch > 'Z') )
		ch = ' ';
	ch -= ' ';
	return(ch );
}
