// HSCWGen.cpp: implementation of the CHSCWGen class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "pathsim.h"
#include "HSCWGen.h"
#include <math.h>
#include "tables.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define K_2PI ( 8.0 * atan(1.0) )		// 2 Pi
#define SECPERSAMPLE (.000125)
#define SYM_OFF 0
#define SYM_ON 1

//#define TX_CONSTANT 10000.0		// TX Amplitude Factor
#define TX_CONSTANT 20000.0		// TX Amplitude Factor


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHSCWGen::CHSCWGen()
{

}

CHSCWGen::~CHSCWGen()
{

}

void CHSCWGen::Init(double CarFreq, double Fs, double Lpm, CString Text)
{
	m_CarrierPhaseInc = CarFreq*K_2PI/Fs;
	m_SecPerSamp = 1.0/Fs;
	m_BitPeriod = 6.0/Lpm;

INT	tmp = (INT)(m_BitPeriod/SECPERSAMPLE);	//round to neares cycle
	m_BitPeriod = (double)tmp * SECPERSAMPLE;

	m_CWState = 0;
	m_time = 0.0;
	m_BitTime = 0.0;
	m_Lastsymb = 0;
	m_CWtimer = 0;
	m_TxShiftReg = 0;
	m_ModState = 0;
	if( Lpm<=250 )
		LPiir.Init(IIR_LPCOEF100);
	else if( Lpm<=1250 )
		LPiir.Init(IIR_LPCOEF500);
	else if( Lpm<=2500 )
		LPiir.Init(IIR_LPCOEF1000);
	else
		LPiir.Init(IIR_LPCOEF1500);
	m_TextString = Text;
	m_TextLength = m_TextString.GetLength();
	m_TextPos = 0;
}

/////////////////////////////////////////////////////////////
// generates n samples of CW waveform in data array pData
/////////////////////////////////////////////////////////////
void CHSCWGen::CalcCW(double *pData, INT n)
{
INT i;
double Mod;
	for( i=0; i<n; i++ )		//calculate n samples of tx data stream
	{
//		Mod = LPiir.CalcFilter(	m_ModState );
		Mod = (double)m_ModState;
		m_time += m_CarrierPhaseInc;			// increment radian phase count
		pData[i] = TX_CONSTANT*Mod*sin( m_time );
		m_BitTime += m_SecPerSamp;
		if( m_BitTime >= m_BitPeriod )//if time to update bit 
		{
			m_BitTime -= m_BitPeriod;			//keep time bounded
			m_time = fmod(m_time,K_2PI);		//keep radian counter bounded
			m_ModState = GetNextCWSymbol();
			if(m_ModState != m_LastModState)
				m_time = 0.0;
			m_LastModState = m_ModState;
		}
	}
}


/////////////////////////////////////////////////////////////
// called every symbol time to get next CW symbol and get the
// next character from the character Queue if no more symbols
// are left to send.
/////////////////////////////////////////////////////////////
INT CHSCWGen::GetNextCWSymbol(void)
{
INT symb;
INT ch;
	symb = m_Lastsymb;		//use last symbol unless it needs to change
	if( (m_TxShiftReg == 0 ) && (m_CWState == 0) )
	{
		ch = GetChar();			//get next character to xmit
		if( ch >=0 )			//if is not a control code
		{
			ch &= 0xFF;		
			ch = (INT)toupper( (BYTE)ch );	//make upper case
			if( ch>=' ' && ch<='Z')
				m_TxShiftReg = CW_TABLE[ ch-' '];	//look up pattern
		}
		m_CWState = 0;
	}
	switch( m_CWState )		// CW timing state machine
	{
		case 0:			//get next cw symbol state from pattern
			switch( m_TxShiftReg&0xC000 )
			{
				case 0x4000:	//dot
					m_CWState = 1;
					m_CWtimer = 1;
					symb = SYM_ON;
					break;
				case 0x8000:	//dash
					m_CWState = 1;
					m_CWtimer = 3;
					symb = SYM_ON;
					break;
				case 0xC000:	//inter char space 
					m_CWState = 1;
					m_CWtimer = 1;
					symb = SYM_OFF;
					break;
				default:
					symb = SYM_OFF;
					break;
			}
			m_TxShiftReg = m_TxShiftReg<<2;
			break;
		case 1:		//On time state
			if( --m_CWtimer <= 0 )
			{
				m_CWtimer = 0;	//inter symbol timer
				m_CWState = 0;
				symb = SYM_OFF;
			}
			else
				symb = SYM_ON;
			break;
		case 2:		//Off time state
			if( --m_CWtimer <= 0 )
				m_CWState = 0;
			break;
	}
	m_Lastsymb = symb;
	return symb;
}



INT CHSCWGen::GetChar()
{
INT ch;
	if( m_TextPos >= m_TextLength)
		m_TextPos = 0;
	ch = m_TextString.GetAt(m_TextPos++);
	return (ch & 0x7F);
}
