// Path.cpp: implementation of the CPath class.
//
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@mindspring.com>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "pathsim.h"
#include "Path.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


#define K_2PI ( 8.0 * atan(1.0) )		// 2 Pi
#define FREQ_CONST (K_2PI/8000.0)		//2Pi/8000
#define SECPERSAMP (1.0/8000.0)

#define MAINSAMPRATE 8000
#define TRIGSAMPRATE (100)
#define FALLTORISE	25.0	//ratio of fall over rise time constants
#define RISEEXPLEN	4.0	//number of time constants for rise exponential length
#define FALLEXPLEN	3.5		//number of time constants for fall exponential length

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPath::CPath()
{
}

CPath::~CPath()
{
}

//////////////////////////////////////////////////////////////////////
//  Initialize a path
//////////////////////////////////////////////////////////////////////
void CPath::InitPath( INT Epm, INT Amp, INT Length, double DopShift,
					 INT blocksize, BOOL active, BOOL RandEpm,
					 BOOL RandAmp, BOOL RandLenght, BOOL RandDop)
{
time_t tm;
	time( &tm );
	Rndgen.Init(tm);
	m_BlockSize = blocksize;
	m_PathActive = active;
	m_Epm = (8000*60)/Epm;	//convert to sample rate units
	m_RandTrigThreshold = (INT)Rndgen.Exprand(m_Epm);
	m_Length = (double)Length/1000.0;	//mSecs to seconds
	m_TriggerCounter = 0;
	m_EventActive = FALSE;
	m_UseRandEPM = RandEpm;
	m_UseRandAmp = RandAmp;
	m_UseRandLength = RandLenght;
	m_UseRandDop = RandDop;
	m_EventAmp = (double)Amp;
	m_EventDoppler = (double)DopShift;

}

//////////////////////////////////////////////////////////////////////
// Performs a path calculation on pIn and puts it in pOut
//
//////////////////////////////////////////////////////////////////////
INT CPath::CalcPath(_complex *pIn, _complex *pOut)
{
INT i;
INT trigpt = -1;
	if(m_PathActive)		// if this path is active
	{
		for(i=0; i<m_BlockSize; i++)
		{
			if( CheckForEvent() && (trigpt<0) )
				trigpt = i;
			if(	m_EventActive )
			{
				m_EventActive = ProcessEvent( pIn[i], pOut[i] );
			}
			else
			{
				pOut[i].x = 0.0;
				pOut[i].y = 0.0;
			}
			m_TriggerCounter++;
		}
	}
	else		// if path is not active just zero the output
	{
		for(i=0; i<m_BlockSize; i++)
		{
			pOut[i].x = 0.0;
			pOut[i].y = 0.0;
			m_TriggerCounter = 0;
		}
	}
	return trigpt;
}

BOOL CPath::ProcessEvent(_complex In, _complex &Out)
{
double Gn;
_complex tmp;
_complex dopshift;
	if(m_ETime <= m_MaxAmpTime )
	{
		Gn = m_Amax - m_Amax*exp(-m_ETime/m_EventT1);
	}
	else
	{
		Gn = m_Amax*exp(( m_MaxAmpTime - m_ETime)/m_EventT2);
	}

	tmp.x = Gn*In.x;
	tmp.y = Gn*In.y;
//
	dopshift.x = cos(m_DopTimeinc);		//Cpx multiply by offset frequency
	dopshift.y = sin(m_DopTimeinc);
	Out.x = ((dopshift.x*tmp.x) - (dopshift.y*tmp.y));
	Out.y = ((dopshift.x*tmp.y) + (dopshift.y*tmp.x));
	m_DopTimeinc += m_DopInc;
	m_DopInc += m_FRate;
	m_DopTimeinc = fmod(m_DopTimeinc,K_2PI);	//keep radian counter bounded
	if(m_ETime <= m_EventLength)
	{
		m_ETime += SECPERSAMP;
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

void CPath::CalcEventMetrics()
{
double tmp;
	if(m_UseRandAmp)
		tmp = GetRandom(-m_EventAmp, 0);
	else
		tmp = -m_EventAmp;
	m_Amax = pow(10, tmp/20.0);
	if(m_UseRandLength)
		m_EventLength = GetRandom( 0.01, m_Length);
	else
		m_EventLength = m_Length;
	tmp = SECPERSAMP*m_EventDoppler/m_EventLength;
	if(m_UseRandDop)
		m_FRate = GetRandom(-tmp, tmp)*FREQ_CONST;
	else
		m_FRate = tmp*FREQ_CONST;	//radian doppler shift rate
	m_DopInc = 0.0;
	m_MaxAmpTime = m_EventLength/(FALLTORISE+1.0);
	m_EventT2 = (m_EventLength - m_MaxAmpTime)/FALLEXPLEN;
	m_EventT1 = m_MaxAmpTime/RISEEXPLEN;
	m_ETime = 0.0;
}

double CPath::GetRandom(double min, double max)
{
double x;
	x = (double)rand();	//0.0 to RAND_MAX uniform random number
	return min + ((max-min)*x)/(double)RAND_MAX;
}

BOOL CPath::CheckForEvent()
{
BOOL ret = FALSE;
	if(m_UseRandEPM)
	{
		if( m_TriggerCounter > m_RandTrigThreshold )
		{
			m_RandTrigThreshold = (INT)Rndgen.Exprand(m_Epm);
			m_TriggerCounter = 0;
			CalcEventMetrics();
			m_EventActive = TRUE;
			ret = TRUE;
gDebug1 += 1.0;
		}
	}
	else
	{
		if( m_TriggerCounter > m_Epm )
		{
			m_TriggerCounter = 0;
			CalcEventMetrics();
			m_EventActive = TRUE;
			ret = TRUE;
gDebug1 += 1.0;
		}
	}
	return ret;
}
