// Rand.cpp: implementation of the CRand class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "pathsim.h"
#include "Rand.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define IA 16807L
#define IM 2147483647L
#define AM (1.0/IM)
#define IQ 127773L
#define IR 2836L
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMAX (1.0-EPS)


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRand::CRand()
{
	Init(123456);
}

CRand::~CRand()
{

}

void CRand::Init(long seed)
{
unsigned long int k;
	if( seed <=0)
		m_idum = 1;
	else
		m_idum = seed;
	for(INT j=NTAB+7; j>=0;j--)
	{
		k = m_idum/IQ;
		m_idum = IA*(m_idum - k*IQ) - IR*k;
		if(m_idum < 0)
			m_idum += IM;
		if( j < NTAB )
			m_iv[j] = m_idum;
	}
	m_iy = m_iv[0];
}

double CRand::Urand()
{
INT j;
long k;
double temp;
	k = m_idum/IQ;
	m_idum = IA*(m_idum-k*IQ)-IR*k;
	if(m_idum<0)
		m_idum += IM;
	j = m_iy/NDIV;
	m_iy = m_iv[j];
	m_iv[j] = m_idum;
	if( (temp=AM*m_iy) > RNMAX)
		return RNMAX;
	else
		return (temp);
}


//  Returns an exponentially distributed, positive, random deviate
// with a mean of 1.0
double CRand::Exprand(double lambda)
{
double tmp;
	do
	{
		tmp = Urand();
	}while(tmp == 0.0);		// get a non zero uniform number
	tmp = -lambda*log(tmp);
	return tmp;
}


